// Copyright 1996, Marimba Inc. All Rights Reserved.
// Please feel free to use this code as an example.

import java.io.*;
import java.awt.*;
import java.net.*;
import java.util.*;

import marimba.io.*;
import marimba.gui.*;
import marimba.persist.*;
import marimba.desktop.Desktop;
import marimba.channel.ApplicationContext;

/**
 * A widget for rotating ads. This widget reads a file containing
 * a list of advertisements. The ads are rotated randomly and a record
 * is kept of how often each ad has been shown to the user. When the widget
 * is stopped the statistics are logged so that they eventually they will
 * be accumulated on transmitter by a plugin.
 *
 * @author	Arthur van Hoff
 * @author	Jonathan Payne
 */
public class AdWidget extends Widget implements ClockClient {
    String src;
    int delay = 60;

    Vector urls;
    Vector imgs;
    Vector keys;

    Hashtable counts;

    int nads;
    int ypos;
    Image oldimg;
    Image img;
    URL base;
    int current = -1;

    ApplicationContext context;

    /**
     * Get the properties of this widget.
     */
    public void getProperties(PropertyList list) {
	super.getProperties(list);
	list.setURL("src", src, null);
	list.setInteger("delay", delay, 60);
    }

    /**
     * Set the properties of this widget.
     */
    public void setProperties(PropertyList list) {
	super.setProperties(list);
	src = list.getURL("src", null);
	delay = list.getInteger("delay", 60);
    }

    /**
     * Load the list of ads and related information
     */
    boolean load(String src) {
	try {
	    nads = 0;
	    imgs = new Vector();
	    keys = new Vector();
	    urls = new Vector();
	    base = getPresentation().getURL(src);

	    FastInputStream in = new FastInputStream(base.openStream());
	    String str;

	    while ((str = in.readLine()) != null) {
		str = str.trim();
		if ((str.length() >= 0) && !str.startsWith("#")) {
		    StringTokenizer t = new StringTokenizer(str);
		    if (t.countTokens() >= 3) {
			imgs.addElement(t.nextToken());
			keys.addElement(t.nextToken());
			urls.addElement(t.nextToken());
		    }
		}
	    }
	    in.close();

	    nads = imgs.size();
	    return true;
	} catch (MalformedURLException e) {
	    e.printStackTrace();
	    return false;
	} catch (IOException e) {
	    e.printStackTrace();
	    return false;
	}
    }

    /**
     * Load listing and initialize counts
     */
    public void start() {
	img = null;
	current = -1;
	counts = null;
	context = (ApplicationContext)getPlayerPanel().getContext();
	if ((delay > 0) && load(src)) {
	    pick();
	    if (oldimg == null) {
		ypos = height;
		Clock.master.add(this, System.currentTimeMillis() + delay*1000, null);
	    } else {
		Clock.master.add(this);
	    }
	} else {
	    repaint();
	}
    }

    /**
     * Advance the animation.
     */
    public long tick(long tm, Object arg) {
	if (ypos > height) {
	    if (oldimg == null) {
		pick();
		return tm;
	    }

	    oldimg = null;
	    repaint();
	    clearPopups();
	    return System.currentTimeMillis() + delay*1000;
	}
	ypos += 2;
	repaint();
	return System.currentTimeMillis() + 100;
    }

    /**
     * Log the counts of how often each ad has been shown.
     */
    public void stop() {
	Clock.master.remove(this);

	if (counts != null && context != null) {
	    for (Enumeration e = counts.keys() ; e.hasMoreElements() ;) {
		Object key = e.nextElement();
		context.appendLog("count " + key + " " + counts.get(key));
	    }
	}
    }

    /**
     * Restart the widget. This method is called by the main program
     * when new data (and possibly new ads) is available.
     */
    void restart() {
	stop();
	start();
    }

    /**
     * Pick the next advertisement.
     */
    void pick() {
	int i = (int)(Math.random() * nads);
	while (current != i) {
	    current = (int)(Math.random() * nads);
	}
	setImage(i);
    }

    /**
     * Set the image.
     */
    public void setImage(int i) {
	ypos = 0;
	oldimg = img;
	current = i;
	img = null;

	if (counts == null) {
	    counts = new Hashtable();
	}
	Object key = keys.elementAt(i);
	Integer count = (Integer)counts.get(key);
	if (count == null) {
	    count = new Integer(1);
	} else {
	    count = new Integer(count.intValue() + 1);
	}
	counts.put(key, count);

	try {
	    String imgsrc = (String)imgs.elementAt(current);
	    Image newimg = ImageCache.get(new URL(base, imgsrc));
	    if (Toolkit.getDefaultToolkit().prepareImage(newimg, -1, -1, this)) {
		img = newimg;
	    }
	} catch (MalformedURLException e) {
	    e.printStackTrace();
	}
	repaint();
    }

    /**
     * Update the image.
     */
    public boolean imageUpdate(Image newimg, int flags, int x, int y, int w, int h) {
	if ((flags & ALLBITS) != 0) {
	    img = newimg;
	    repaint();
	    return false;
	}
	return (flags & ERROR) == 0;
    }

    /**
     * Paint the image.
     */
    public void paint(Graphics g) {
	if (oldimg != null) {
	    int w = oldimg.getWidth(this);
	    int h = oldimg.getHeight(this);
	    if ((w > 0) && (h > 0)) {
		Graphics gc = g.create();
		try {
		    gc.clipRect(0, ypos, width, height - ypos);
		    gc.drawImage(oldimg, (width - w) / 2 , (height - h) / 2, this);
		} finally {
		    gc.dispose();
		}
	    }
	}
	if (img != null) {
	    int w = img.getWidth(this);
	    int h = img.getHeight(this);
	    if ((w > 0) && (h > 0)) {
		if (ypos < height) {
		    g.clipRect(0, 0, width, ypos);
		}
		g.drawImage(img, (width - w) / 2 , (height - h) / 2, this);
	    }
	}
    }   

    /**
     * Handle mouse events.
     */
    public boolean handleEvent(Event evt) {
	switch (evt.id) {
	  case Event.MOUSE_DOWN:
	    action();
	    return true;
	}
	return super.handleEvent(evt);
    }

    /**
     * The user has clicked in the image.
     */
    public void action() {
	if (current >= 0) {
	    String url = (String)urls.elementAt(current);
	    if (context != null) {
		context.appendLog("link " + url);
	    }

	    // Show the ad URL in the browser.
	    Desktop.getDesktop().showURL(getPresentation().getURL(url));
	}
    }

    /**
     * Get the tip for the popup window.
     */
    public String getTip() {
	return (current >= 0) ? "go to " + urls.elementAt(current) : null;
    }
}
