// Copyright 1996, Marimba Inc. All Rights Reserved.
// @(#)ApplicationContext.java, 1.18, 01/13/97

package marimba.channel;

import java.net.*;
import java.util.*;
import java.io.IOException;
import java.applet.AudioClip;
import java.awt.Image;

import marimba.io.*;

/**
 * This provides the context for an Application. It is the interface
 * of the application to the tuner.
 *
 * @author	Arthur van Hoff
 * @author	Maurice Balick
 * @version 	1.18, 01/13/97
 */
public interface ApplicationContext {
    /**
     * Get the base URL of this channel.
     */
    URL getBase();

    /**
     * Get the code base URL of this channel. This is where
     * the code for the channel is stored.
     */
    URL getCodeBase();

    /**
     * Get the base URL for the data directory of this channel.
     * This is where the directory where the channel can store
     * persistent data.
     * @see getDataDirectory
     */
    URL getDataBase();
    
    /**
     * Returns true if the specified path exists for this channel.  Path
     * must be a relative pathname.
     */
    boolean channelFileExists(String path);

    /**
     * Returns an array of strings which are the names of files in
     * the specified directory.  Directory is a relative path.  To
     * list the root of a channel, directory should be "".
     *
     * Returns null if directory does not exist or is not a directory.
     */
    String[] listChannelDirectory(String directory);

    /**
     * Gets the base data directory for this channel. This directory
     * can be used to store persistent data. It also contains logging
     * and profiling data. An un authenticated channel is not allowed
     * to write anywhere else in the file system.
     */
    String getDataDirectory();

    /**
     * Gets the UpdateReport for this channel.  The update report can
     * be queried for a list of all the pending changes in this
     * channel, e.g., a list of the deletions, creations and updates
     * that are pending.
     *
     * Returns null if there are no existing updates of any kind
     * pending on this channel.
     * @see Updates
     */
    Updates getPendingUpdates();

    /**
     * OBSOLETE. Will be removed in a future release
     * @see getBase
     * @see channelFileExists
     * @see listChannelDirectory */
    String getChannelDirectory();

    /**
     * Get the name of the server of this channel.
     * The server name is of the form: "hostname:port".
     */
    String getServerName();

    /**
     * Get the name of this channel.
     */
    String getChannelName();

    /**
     * Get an application parameter. The application parameters are
     * specified in a properties file in the application directory.
     * Application parameters are case-insensitive.
     */
    String getParameter(String nm);

    /**
     * Request the installation of new all new data or just part
     * in the given directory. Pass an empty string to install
     * the entire channel.
     */
    void installData(String dir);

    /**
     * Logs an array of bytes for the channel.  The log data is sent
     * back to the transmitter at some point in the future, usually
     * when the next update is requested. Returns false if an error
     * occurred while creating the log entry.
     */
    boolean appendLog(byte data[]);

    /**
     * Logs a String for the channel.  The log data is sent back to
     * the transmitter at some point in the future, usually when the
     * next update is requested. Returns false if an error occurred
     * while creating the log entry.
     */
    boolean appendLog(String data);

    /**
     * Get the profile data associated with this channel. This data is
     * included in every request to the transmitter and can be used
     * for profiling. This will return null an error occurred while
     * reading the profile. It will return an empty profile if no
     * profile was created yet.
     */
    byte[] getProfile();

    /**
     * Set the profile for a channel. This data is included in every
     * request to the transmitter and can be used for profiling. This
     * will the override the previous profiling data (if any). The profile
     * data is application specific and can be any format you like.
     * The profile data can be deleting by passing in null. Returns
     * false if the profile could not be saved.
     */
    boolean setProfile(byte data[]);

    /**
     * Return the time at which this version of the
     * channel was published.
     */
    long publishTime();

    /**
     * Return the time the channel was updated with new data.
     */
    long updateTime();

    /**
     * Restart the application. The application can call this method
     * after recieving a notifyAvailable() call. As a result the
     * channel is stopped and restarted with the new changes.
     */
    void restart();

    /**
     * Stop the application. The application should call this
     * methods to notify the tuner that it wants to be destroyed.
     * A call to stop() and destroy() will follow shortly.
     */
    void stop();

    /**
     * Update the application. The application can call this to request
     * an update to occur. This will cause the tuner to contact
     * the transmitter (which will happen automatically some
     * small amount of time after this call) to find out if
     * an update is needed.
     */
    void update();

    /**
     * Start a channel.
     */
    void startChannel(String serverName, String channelName);

    /**
     * Subscribe to a channel, but don't start it yet.
     */
    void subscribeChannel(String serverName, String channelName);

    /**
     * Unsubscribe a channel from this transmitter
     */
    void unsubscribeChannel(String channelName);

    /**
     * Remove a channel from this transmitter
     */
    void removeChannel(String channelName);

    /**
     * Get a list of all the channels on this transmitter
     * which have been subscribed to by the user. 
     */
    String[] listChannels();

    /**
     * Get the status of a channel on this transmitter.
     * The status will be "unsubscribed", "subscribed",
     * or "running".
     */
    String getChannelStatus(String channelName);

    /**
     * Get an audio clip.
     */
    AudioClip getAudioClip(URL ur);

    /**
     * Get an image.
     */
    Image getImage(URL url);

    /**
     * Show a document.
     */
    void showDocument(URL url);

    /**
     * Show a document in a named frame.
     */
    void showDocument(URL url, String frame);

    /**
     * Show a document.
     */
    void showDocument(String url);

    /**
     * Show a Document in a frame.
     */
    void showDocument(String url, String frame);

    /**
     * Show status.
     */
    void showStatus(String msg);
}
